import { EventEmitter2 } from 'eventemitter2';
import { Map, Set } from 'immutable';
import { Observable } from 'rxjs';
import { LocalMediaHandler } from '../media/LocalMediaHandler';
import { ManagedStream } from '../media/ManagedStream';
import { Dtmf, MediaTypes } from '../types';
import { Conference as StackConference } from '../wac-proxy/stacks/Janus/Conference';
import { ConferenceInvite } from './ConferenceInvite';
import { ConferenceStatus } from './ConferenceStatus';
import { Invitation } from './Invitation';
import { Participant } from './Participant';
/**
 * @typedef {Object} EventInfo
 * @property {string} participant gatewayUsername of a participant in the conference
 * @property {string} [reason='IN_CALL'|'HOLD'] reason to know if the participant is on hold or not
 */
/**
 * @typedef {Object} RemoteStreamAddedEvent
 * @property {string} participant gatewayUsername of the local stream
 * @property {string} [reason='NEW'|'UNHOLD'] reason of stream addition
 * @property {@link ManagedStream} stream
 */
/**
 * @typedef {Object} RemoteStreamRemovedEvent
 * @property {string} participant gatewayUsername of the local stream
 * @property {string} [reason='GONE'|'HOLD'] reason of stream addition
 * @property {@link ManagedStream} stream
 */
/**
 * @typedef {Object} SpeakerChangedEvent
 * @property {string} participant gatewayUsername of the conference participant
 * @property {boolean} isTalking indicates the new value for the associated participant
 */
/**
 * Provides access to methods for managing outgoing or incoming conferences.
 * {@link Conference} objects are obtained calling {@link ConferenceManager#createConference} method or handling
 * the `onIncoming` event of a connected {@link ConferenceManager} instance.
 *
 * ## Events
 *
 * - `remoteStreams` ({@link ImmutableSet}<{@link ManagedStream}> currentValue, {@link ImmutableSet}<{@link ManagedStream}> oldValue, {@link EventInfo})
 *    - Emitted every time remoteStreams changes.
 * - `outgoingInvitations` ({@link ImmutableSet}<{@link Invitation}> currentValue, {@link Invitation}<{@link ManagedStream}> oldValue)
 *    - Emitted every a outgoingInvitation changes.
 * - `status` ({@link ConferenceStatus} currentValue, {@link ConferenceStatus}<{@link ManagedStream}> oldValue)
 *    - Emitted every time the conference status changes.
 *      deprecated
 * - `remoteParticipants` ({@link Map}<{@link gatewayUsername}, 'IN_CALL'|'HOLD'> currentValue, {@link Map}<{@link gatewayUsername}, 'IN_CALL'|'HOLD'> oldValue)
 *   - Emitted every time the participants list changes.
 * - `transferError`
 *   - Emitted when an error occurs in a transfer.
 * - `participantAdded` ({@link gatewayUsername} gatewayUsername)
 *   - Emitted when a new participant joins the conference
 * - `participantRemoved` ({@link gatewayUsername} gatewayUsername)
 *   - Emitted when a participant leaves the conference
 * - `participantHold` ({@link gatewayUsername} gatewayUsername)
 *   - Emitted when a participant starts holding the call
 * - `participantUnhold` ({@link gatewayUsername} gatewayUsername)
 *   - Emitted when a participant stops holding the call
 * - `remoteStreamAdded` ({@link RemoteStreamAddedEvent})
 *   - Emitted on remote stream added
 * - `remoteStreamRemove` ({@link RemoteStreamRemovedEvent})
 *   - Emitted on remote stream removed
 * - `speakerChanged`({@link SpeakerChangedEvent})
 *   - Emitted each time a conference participant starts or stops talking
 */
export declare class Conference {
    emitter: EventEmitter2;
    participants$: Observable<Record<string, Participant>>;
    muteRequest$: Observable<boolean>;
    /** Use to keep the number of invitations sent in this conference */
    invitationCount: number;
    private room;
    private status;
    private localMediaHandler;
    private transferingCall;
    private outgoingInvitations;
    private remoteParticipants;
    private remoteStreams;
    private joinTimestamp;
    private subscription;
    private constructor();
    static create(room: StackConference, mediaConstraints: MediaStreamConstraints): Promise<Conference>;
    private getActions;
    private bindEventHandlers;
    private unbindEventHandlers;
    private onLocalStreamChanged;
    private onRemoteStreamAdded;
    private getRemoteStream;
    private onRemoteStreamRemoved;
    private onRemoteStreamChanged;
    private onExpelled;
    private onDestroyed;
    private onTransferingCall;
    private updateRemoteParticipants;
    private onParticipantAdded;
    private onParticipantRemoved;
    private onParticipantHold;
    private onParticipantUnhold;
    private onSpeakerChanged;
    /**
     * Manages the Invitations Set and events.
     */
    private updateOutgoingInvitations;
    private getOutgoingInvitation;
    private hasActiveInvitation;
    /**
     * This is assuming that if it does not has an invitation answered 200, it could never been in the room
     * Made to solve PSTN problems (users with early media cannot be re-invited)
     * TODO We need a proper way to check if the user is already joined
     */
    private alreadyJoined;
    private canSendInvitation;
    /**
     * Returns if the conference is recorded
     */
    isRecorded(): boolean;
    /**
     * Returns the ID of the conference
     */
    getId(): string;
    /**
     * Returns the remote participants
     */
    getRemoteParticipants(): Map<string, 'HOLD' | 'IN_CALL'>;
    /**
     * Returns the current status of the conference
     */
    getStatus(): ConferenceStatus;
    /**
     * Returns an observable for the status of the conference
     */
    getStatus$(): Observable<ConferenceStatus>;
    /**
     * Allows access to local streams and actions related to changing which local media is shared
     */
    getLocalMediaHandler(): LocalMediaHandler;
    /**
     * Returns a Map containing a list of ManagedStreams per participant (gateway user name)
     */
    getRemoteStreams(): Map<string, ManagedStream[]>;
    /**
     * Returns a Set of strings containing the gateway username of every remote participant
     * @deprecated
     */
    getRemoteGatewayUsernames(): Set<string>;
    /**
     * Returns a Set of outgoing invitations
     */
    getOutgoingInvitations(): Set<Invitation>;
    /**
     * Check if a conference is being transfering
     */
    isBeingTransfered(): boolean;
    /**
     * Returns a boolean value indicating if at least one of the remote conference participants
     * has a video track
     */
    hasRemoteVideo(): boolean;
    private setStatus;
    /** @deprecated This property always remove false */
    isRecovered(): boolean;
    /**
     * Returns the current participant count.
     * Includes in the count the users with active invitations.
     */
    getCurrentParticipants(): number;
    /**
     * Returns the invitation to the conference if the participant was invited.
     */
    getInvite(): ConferenceInvite | undefined;
    /**
     * Invites a new participant to this conference
     * @throws {Error} throw error when a participant has already invited or is joined to the conference.
     * @param participant A participant can be both an internal or an external user.
     * @param mediaTypes the MediaTypes of this invitation
     * @param Arbitrary context that can be sent when making an invite
     * @returns A promise with an object with two properties: SIP code (SIP code of the response) and
     * whoAccepted (User id of the participant that accepted the invite).
     */
    inviteParticipant(participant: string, mediaTypes?: MediaTypes, context?: Record<string, string>): Promise<{
        code: number;
        whoAccepted?: string;
    }>;
    /**
     * Invites many participants to a conference.
     * @param participants
     * @param mediaTypes the MediaTypes for this invitation
     * @param context Arbitrary context that can be sent when making an invite
     * @returns A promise with an array of invitation objects
     * waitUntilResolve Will resolve once the invitation is answered.
     * The resolution will have two fields: code (a SIP code) and whoAccepted (userId of the participant who accepted)
     */
    inviteMany(participants: string[], mediaTypes?: MediaTypes, context?: Record<string, string>): Promise<Array<{
        waitUntilResolved: () => Promise<{
            code: number;
            whoAccepted?: string;
        }>;
    }>>;
    private updateOutgoingResolvedInvitationsForParticipant;
    /**
     * Expels a participant from this conference
     */
    expelParticipant(participant: string, session?: string): Promise<void>;
    /**
     * Cancels a invitation of this conference
     */
    cancelInvitation(invitation: Invitation, session?: string): void;
    /**
     * Invites a list of new participant to this conference
     */
    inviteParticipants(participants: string[]): Promise<Array<{
        code: number;
        whoAccepted?: string | undefined;
    }>>;
    /**
     * Expels a list of participant from this conference
     */
    expelParticipants(participants: string[]): Promise<void>;
    /**
     * Connects to the conference
     */
    join(): Promise<void>;
    /**
     * Invites an agent to a conference
     * @param mediaTypes the media types that will be used to call an agent
     * @param contextInfo arbitrary context that can be sent to an agent
     * @param token third-party call ID.
     */
    callAgent(mediaTypes?: MediaTypes, contextInfo?: Record<string, string>, token?: string): Promise<{
        code: number;
        whoAccepted?: string | undefined;
    }>;
    /**
     * Invites an agent with a particular skill to a conference
     * @param mediaTypes the media types that will be used to call an agent
     * @param skill the skill an agent must have in order to be invited
     * @param contextInfo arbitrary context that can be sent to an agent
     * @param token third-party call ID.
     */
    callSkilledAgent(mediaTypes?: MediaTypes, skill?: string, contextInfo?: Record<string, string>, token?: string): Promise<{
        code: number;
        whoAccepted?: string | undefined;
    }>;
    /**
     * Invites the voicemail to a conference
     * @param mediaTypes the media types that will be used to call the voicemail
     */
    callVoiceMail(mediaTypes?: MediaTypes): Promise<{
        code: number;
        whoAccepted?: string | undefined;
    }>;
    /**
     * Returns the timestamp when the join to this conference was completed
     * @returns timestamp when the join was completed
     */
    getJoinTimestamp(): number | undefined;
    /**
     * Leaves the conference
     * @param errorCode the rejection cause when an invitation is received
     * and the conference has not been joined.
     */
    leave(errorCode?: number): Promise<void>;
    /**
     * Local hold
     * Stop publishing and subscribing to the media in the conference
     */
    hold(): Promise<void>;
    /**
     * Stop local hold
     * Republishes all previously published media, subscribes to previously subscribed
     * media and notifies new status to other participants.
     */
    unhold(): Promise<void>;
    /**
     * Check if conference is on hold state.
     */
    get isOnHold(): boolean;
    /**
     * Create an unattendedCallTransfer towards a user in the system
     * @param participant
     * @param mediaTypes the current media types will be used if nothing is specified
     */
    unattendedCallTransfer(participant: string, mediaTypes?: MediaTypes): Promise<void>;
    /**
     * Send data to the audio's room
     */
    sendDtmf(value: Dtmf): Promise<void>;
    muteRequest(to: string, muted?: boolean): Promise<void>;
}
